package com.urbancode.air.plugin.rtw

import com.urbancode.air.CommandHelper

public class RTWProductHelper {
	
	def ENV_VAR_SDP_DIR = "TEST_WORKBENCH_HOME"; //$NON-NLS-1$
	// Mandatory Options for Test Execution
	def workspace ;
	def project ;
	def suite ;
	
	// Read non-mandatory fields
	def imsharedloc ;
	def varfile ;
	def configfile;
	def results ;
	def overwrite ;
	def quiet ;
	def users ;
	def vmargs ;
	def exportlog ;
	def exportstats ;
	def exportstatreportlist ;
	def usercomments ;
	
	def product = RTWProduct.RTWec;
	def eclipsehome = "";
	def plugins="";
	
	def cmdHelper = new CommandLineHelper();
	
	RTWProductHelper(def props,RTWProduct prod) {
		printProps(props);
		product = prod;
		
		workspace = props['workspace'];
		project = props['project'];
		suite = props['suite'];
		
		// Read non-mandatory fields
		imsharedloc = props['imsharedloc'];
		varfile = props['varfile'];
		configfile = props['configfile'];
		results = props['results'];
		overwrite = props['overwrite'];
		quiet = props['quiet'];
		if(prod==RTWProduct.RPT){
			// We may have to remove users and usercomments from the UI for RTWec test
			// whenever I passed users from command line it threw NullpointerException and comments 
			// cannot be seen anywhere. Discussed with Siva.
			users = props['users'];
			usercomments = props['usercomments'];
		}
		vmargs = props['vmargs'];
		exportlog = props['exportlog'];
		exportstats = props['exportstats'];
		exportstatreportlist = props['exportstatreportlist'];
		
		
	}
	
	final def workDir = new File('.').canonicalFile
	
	def printProps(def props){
		println "Working Directory = "+workDir;
		
		println "Printing Step Properties ....";
		
		def propKeys = props.keys();
		while(propKeys.hasMoreElements()){
			def key = propKeys.nextElement();
			def val = props.get(key);
			println key+" = "+val;
		}
	}
	
	
	def validateInputs(){
		
		try{
			// First validate all values input by the User
			def fWorkspace = new File(workspace);
			if (!fWorkspace.exists()) {
				throw new RuntimeException("Invalid Workspace location :"+workspace);
			}
			
			def projectPath = workspace + File.separator + project;
			println "Project Path" + projectPath;
			
			def fProject = new File(projectPath);
			if (!fProject.exists()) {
				throw new RuntimeException("Invalid Project location :"+projectPath);
			}
		
			
			if(varfile.toString().trim().length() > 0){
				def fVarfile = new File(varfile);
				if (!fVarfile.exists()) {
					throw new RuntimeException("Invalid Var File location :"+varfile);
				}
			}
			
			if(configfile.toString().trim().length() > 0){
				def fConfigfile = new File(configfile);
				if (!fConfigfile.exists()) {
					throw new Exception("Invalid Config File location :"+configfile);
				}
			}
			
			if(exportlog.toString().trim().length() > 0){
				def fExportlog = new File(exportlog);
				if (fExportlog.exists()) {
					def delStatus = fExportlog.delete();
					//println "Deleted existing file successfully "+delStatus.toString();
				}
			}else{
				// set default export log file
				exportlog = workDir.toString()+File.separator+"temp"+ File.separator+ suite+".exportlog.log";
			}
			
			if(exportstats.toString().trim().length() > 0){
				def fExportstats = new File(exportstats);
				
				if (!fExportstats.exists()) {
					throw new Exception("Invalid Exported Statistical Report Data File location :"+exportstats);
				}
			}
				
		}catch(Exception ex){
			println "Error:"+ex.message;
			ex.printStackTrace();
			System.exit(1);
		}
	}
	
	
	def runTest(def mainOutProps){
		validateInputs();
		initIMData();
		executeTest(mainOutProps);
	}
	
	
	def executeTest(def mainOutProps){
		try {
			def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()
			def out = System.out;
			def batchFileName = isWindows ? "cmdline.bat" : "cmdline.sh"
			def dirCmdline = eclipsehome + File.separator +"cmdline";
			
			println "cmdline directory path " + dirCmdline;
			def dirBatchFilePath = dirCmdline+File.separator+ batchFileName;

			def fdirBatchFile = new File(dirBatchFilePath);
			if (!fdirBatchFile.exists()) {
				throw new RuntimeException("Invalid Commandline batch file location :"+dirBatchFilePath);
			}
			def ch = new CommandHelper(new File(dirCmdline))

			// Build args
			def cmdArgs = [];
			def arg="";
			cmdArgs = [dirBatchFilePath, '-workspace', workspace, '-project', project, '-plugins', plugins, '-eclipsehome', eclipsehome, '-suite', suite];

			if(varfile != null && !varfile.toString().isEmpty())
			{
				arg = '-varfile';
				cmdArgs << arg;
				cmdArgs << varfile;
			}

			if(configfile != null && !configfile.toString().isEmpty())
			{
				arg = '-configfile';
				cmdArgs << arg;
				cmdArgs << configfile;
			}

			if(vmargs != null && !vmargs.toString().isEmpty())
			{
				arg = '-vmargs';
				cmdArgs << arg;
				cmdArgs << vmargs;
			}


			if(exportlog != null && !exportlog.toString().isEmpty())
			{
				arg = '-exportlog';
				cmdArgs << arg;
				cmdArgs << exportlog;
			}

			if(exportstats != null && !exportstats.toString().isEmpty())
			{
				arg = '-exportstats';
				cmdArgs << arg;
				cmdArgs << exportstats;
			}

			if(exportstatreportlist != null && !exportstatreportlist.toString().isEmpty())
			{
				arg = '-exportstatreportlist';
				cmdArgs << arg;
				cmdArgs << exportstatreportlist;
			}
			
			if(results != null && !results.toString().isEmpty())
			{
				arg = '-results';
				cmdArgs << arg;
				cmdArgs << results;
			}
			
			if(users != null && !users.toString().isEmpty())
			{
				arg = '-users';
				cmdArgs << arg;
				cmdArgs << users;
			}
			 
			if(usercomments != null && !usercomments.toString().isEmpty())
			{
				arg = '-usercomments';
				cmdArgs << arg;
				cmdArgs << usercomments;
			}
			
			if(quiet=="true")
			{
				arg = '-quiet';
				cmdArgs << arg;
				//cmdArgs << quiet;
			}
			
			if(overwrite=="true")
			{
				arg = '-overwrite';
				cmdArgs << arg;
				//cmdArgs << overwrite;
			}

			ch.runCommand("Running test..", cmdArgs);

			def outputProps = new Properties();

			def fResultFile = new File(exportlog);
			if(fResultFile.exists()){

				def exportlogreader = new BufferedReader(new FileReader(exportlog));
				try {
					def sb = new StringBuilder();
					def line = exportlogreader.readLine();

					while (line != null) {
						sb.append(line);
						sb.append("\n");
						line = exportlogreader.readLine();
					}

					def logText = sb.toString();
					def rollupIndex = logText.lastIndexOf("verdict roll-up");
					def status = logText.substring(rollupIndex-6, rollupIndex);
					println "Test Result = "+status;
					outputProps.setProperty("TestResult", status.trim());
				} finally {
					exportlogreader.close();
				}
			}else{
				outputProps.setProperty("TestResult", "FAIL");
			}

			//persist outputProps
			def outputPropsFile = new File(mainOutProps);
			def outputPropsStream = null;
			try {
				outputPropsStream = new FileOutputStream(outputPropsFile);
				outputProps.store(outputPropsStream, null);
			}
			finally {
				outputPropsStream.close();
			}
		}
		catch (Exception e) {
			//println "Error running test!";
			println "Error:"+e.message;
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	
	
	def getTestWorkbenchDir()
	{
		return getDirThroughEnvVar(ENV_VAR_SDP_DIR);
	}
	
	def getDirThroughEnvVar(String envVar) throws RuntimeException
	{
		def value = System.getenv(envVar);
		def isValid = isValidEnvVar(value);
		if(isValid)
		{
			isValid = new File(value).isDirectory();
		}
		
		if(!isValid)
		{
			throw new RuntimeException("Could not find a valid TEST_WORKBENCH_HOME environment variable pointing to installation directory");
		}
		return value;
	}
	
	def isValidEnvVar(def envVarValue)
	{
		
		def valid = true;
		if(envVarValue == null || envVarValue.isEmpty())
			valid = false;
		
		else
		{
			envVarValue = envVarValue.toLowerCase();		
			if(envVarValue.contains("*") || envVarValue.contains("?") || 
					envVarValue.startsWith("del ") || envVarValue.startsWith("rm "))
				valid = false;
		}
		
		return valid;
	}
	
	def getIMSharedLocation(def eclipsehome){
		def ibmloc = null;
		def rollupIndex = eclipsehome.lastIndexOf(File.separator);
		ibmloc = eclipsehome.substring(0, rollupIndex);		
		imsharedloc = ibmloc + File.separator + "HCLIMShared";
		
		return imsharedloc;
	}
	
	
	def initIMData(){
		try {

			eclipsehome = getTestWorkbenchDir();
			println  "eclipsehome "+eclipsehome;
			def fEclipsehome = new File(eclipsehome);
			if (!fEclipsehome.exists()) {
				println "Product eclipse cannot be found at "+ eclipsehome;
				throw new RuntimeException("Product eclipse cannot be found at "+ eclipsehome);
			}

			if (imsharedloc != null && !imsharedloc.equalsIgnoreCase("null") && imsharedloc.toString().trim().length() > 0) {
				println  "imsharedloc : "+imsharedloc;
				def fIbmsharedloc = new File(imsharedloc);
				if(!fIbmsharedloc.exists()){
					throw new RuntimeException("Invalid IMShared Location : "+imsharedloc);
				}
			}
			else {
				imsharedloc = getIMSharedLocation(eclipsehome);
				println  "imsharedloc : "+imsharedloc;
				def fIbmsharedloc = new File(imsharedloc);
				if(imsharedloc == null || imsharedloc.toString().isEmpty() || !fIbmsharedloc.exists()){
					throw new RuntimeException("IMSharedLocation is not at default location, Please provide IMSharedloc parameter");
				}
			}
			
			plugins = imsharedloc + File.separator+"plugins";
			def fIbmshared = new File(plugins);
			if (!fIbmshared.exists()) {
				throw new RuntimeException("IBM Shared Resource Plugins Directory cannot be found at "+ plugins);
			}
			

		}catch (RuntimeException e) {
			// TODO Auto-generated catch block
			throw new RuntimeException(e.toString());
		}catch (Exception e) {
			//System.out.println("Error running test!");
			println "Error:"+e.message;
			e.printStackTrace();
			System.exit(1);
		}

	}

	def getIMSharedLoc(def cmdPath){
		def imsharedLoc = "";
		try {

			// Build args
			def cmdargs = [];
			cmdargs = [cmdPath, 'listInstallationDirectories', '-long'];
			def cmdOutput = cmdHelper.executeAndReturnValue(cmdargs);
			def installresult = cmdOutput;
			def installdirs = installresult.split('\n');
			for (def x=0; x<installdirs.length; x++){
				//println "InstallDir :" + installdirs[x];
				// Path to IBMIMShared
				if(installdirs[x].contains("*")){
					def installtokens = installdirs[x].split(" : ");
					imsharedLoc = installtokens[1];
					println "IBMIMShared Loc "+imsharedLoc;
					break;
				}
			}
		}catch (Exception e) {
			//System.out.println("Error running test!");
			println "Error:"+e.message;
			e.printStackTrace();
			System.exit(1);
		}
		return imsharedLoc;
	}
	
	def srchPkgsForProd(def pkgresult, RTWProduct prod){
		def sdpLoc ="";
		def packages = pkgresult.split("\n");
		for (def x=0; x<packages.length; x++){
			//println "Package :" + packages[x];
			// Path to IBMIMShared
			def pkgtokens = packages[x].split(" : ");
			if(pkgtokens.length > 1){//just a sanity
				if(pkgtokens[1].contains(prod.getFeaturePackage())){// RPT Path
					println "SDP Location "+pkgtokens[0];
					sdpLoc = pkgtokens[0];
					break;
				}
			}
		}
		return sdpLoc;
	}
	
	def getSDPLocation(def cmdPath, RTWProduct prod){
		def sdpLoc = "";
		try {
			def cmdargs = [];
			cmdargs = [cmdPath, 'listInstalledPackages', '-features', '-long'];
			def cmdOutput = cmdHelper.executeAndReturnValue(cmdargs);
			def pkgresult = cmdOutput;
			
			if(pkgresult.contains(prod.getFeaturePackage()))
			{
				sdpLoc = srchPkgsForProd(pkgresult,prod);
			}
			
			if(sdpLoc == "" && prod == RTWProduct.RPT){
				// Try to execute using RST
				if(pkgresult.contains(RTWProduct.RST.getFeaturePackage()))
				{
					sdpLoc = srchPkgsForProd(pkgresult,RTWProduct.RST);
				}
			}
			
			if(sdpLoc == "" && prod == RTWProduct.RTWec){
				// Try to execute using RST
				if(pkgresult.contains(RTWProduct.MTE.getFeaturePackage()))
				{
					sdpLoc = srchPkgsForProd(pkgresult,RTWProduct.MTE);
				}
			}
		
			
		}catch (Exception e) {
			println "Error:"+e.message;
			e.printStackTrace();
			System.exit(1);
		}
		return sdpLoc;
	}
}
